# -*- coding: utf-8 -*-
#
# File: BPDElemento_TraversalConfig.py
## Copyright (c) 2010 by Conselleria de Infraestructuras y Transporte de la
# Generalidad Valenciana
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
#
# Authors: 
# Conselleria de Infraestructuras y Transporte de la Generalidad Valenciana, Model Driven Development sl, Antonio Carrasco Valero
#
#

__author__ = """Conselleria de Infraestructuras y Transporte de la Generalidad Valenciana
<gvSIGbpd@gvSIG.org>, Model Driven Development sl <gvSIGbpd@ModelDD.org>,
Antonio Carrasco Valero <carrasco@ModelDD.org>"""

__docformat__ = 'plaintext'

from AccessControl                  import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.gvSIGbpd.config import *



class BPDElemento_TraversalConfig:            

    """
    """
    security = ClassSecurityInfo()

    security.declarePublic('traversalConfig')
    def traversalConfig( self):
        return [
    {   'portal_types': [ 'BPDArtefacto', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDArtefacto', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                               'codigo',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'estado',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':               'fechaAdopcion',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':               'fechaObsolescencia',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':                               'nivelDeImposicion',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'version',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'caracteristicas',
                'columns':              [ 'title', 'description', 'detallesCaracteristica', 'titulosTiposArtefactos', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesArtefactos',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionArtefactos', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'responsablesDeArtefacto',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'entradasAProcesosDeNegocio',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'esRequerido', 'fuenteDeInformacion', 'valorDefecto', 'titulosArtefactosDeEntrada', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDEntrada', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'salidasDeProcesosDeNegocio',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'esRequerido', 'titulosArtefactosDeSalida', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDSalida', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'pasosQueRecibenElArtefacto',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDRecepcion', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'pasosQueEnvianElArtefacto',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'titulosDestinatarios', 'titulosArtefactosEnviados', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDEnvio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'politicasDeNegocioGobernantes',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioDirigentes',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosGestores',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'usosDelArtefacto',
                'columns':       [ 'title', 'description', 'esRequerido', 'modoDeUso', 'tituloPasoDeProcesoDeNegocio', 'tituloProcesoDeNegocio', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'caracteristicasDelTipo',
                'columns':       [ 'title', 'description', 'claseCaracteristica', 'tituloArtefacto', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDCaracteristica', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDCaracteristica', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'claseCaracteristica',
                'kind':     'Data',
                'type':     'selection',
                'optional': True,
            },
            {   'name':     'multiplicidadMinima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'restriccionTipo',
                'kind':     'Data',
                'type':     'string',
                'optional': True,
            },
            {   'name':     'multiplicidadMaxima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':                               'codigo',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                          'detallesCaracteristica',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                               'estado',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':               'fechaAdopcion',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':               'fechaObsolescencia',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':                               'nivelDeImposicion',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                          'tituloArtefacto',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosTiposArtefactos',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                               'version',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
        ],
        'traversals':   [
            {   'relation_name': 'tiposDeArtefactos',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosGestores',
                'columns':       [ 'title', 'description', 'proposito', 'responsableMantenimiento', 'detallesProceso', 'codigo', 'estado', 'nivelDeImposicion', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'usosDeLaCaracteristica',
                'columns':       [ 'title', 'description', 'esRequerida', 'modoDeUso', 'tituloPasoDeProcesoDeNegocio', 'tituloProcesoDeNegocio', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionArtefactos', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionArtefactos', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'artefactos',
                'columns':              [ 'title', 'description', 'codigo', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionEntradas', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionEntradas', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'entradas',
                'columns':              [ 'title', 'description', 'esRequerida', 'titulosArtefactosDeEntrada', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDEntrada', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionHerramientas', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionHerramientas', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'herramientas',
                'columns':              [ 'title', 'description', 'codigo', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionPasos', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionPasos', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'pasos',
                'columns':              [ 'title', 'description', 'titulosPasosAnteriores', 'titulosPasosSiguientes', 'titulosPasosSiguientesEnCasoExcepcion', 'detallesPaso', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionPerfiles', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionPerfiles', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'perfiles',
                'columns':              [ 'title', 'description', 'abreviatura', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDPerfil', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionPoliticasDeNegocio', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionPoliticasDeNegocio', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'politicasDeNegocio',
                'columns':              [ 'title', 'description', 'codigo', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionProcesosDeNegocio', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionProcesosDeNegocio', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'procedimientos',
                'columns':              [ 'title', 'description', 'proposito', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionReglasDeNegocio', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionReglasDeNegocio', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'reglasDeNegocio',
                'columns':              [ 'title', 'description', 'codigo', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionSalidas', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionSalidas', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'salidas',
                'columns':              [ 'title', 'description', 'esRequerida', 'titulosArtefactosDeSalida', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDSalida', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionUnidadesOrganizacionales', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDColeccionUnidadesOrganizacionales', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'unidadesOrganizacionales',
                'columns':              [ 'title', 'description', 'abreviatura', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDDecision', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDDecision', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'esInicial',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'condicionSiguientePaso',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'detallesPaso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                     'numeroDePaso',
                'kind':                     'Data',
                'type':                     'Number',
                'optional':                 True,
                'exclude_from_values_form': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnteriores',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_views':            [ 'Textual', ],
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientes',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientesEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name':    'pasosSiguientes',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosSiguientesEnCasoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name': 'ejecutores',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosAnteriores',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'requeridoEnPlazos',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDPlazo', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'aggregation_name':     'usosArtefactos',
                'columns':              [ 'title', 'description', 'esRequerido', 'modoDeUso', 'titulosArtefactosUsados', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosCaracteristicas',
                'columns':              [ 'title', 'description', 'esRequerida', 'modoDeUso', 'titulosCaracteristicasUsadas', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDEntrada', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDEntrada', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'esRequerida',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'multiplicidadMinima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'multiplicidadMaxima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'fuenteDeInformacion',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosDeEntrada',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'valorDefecto',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'artefactosDeEntrada',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDEnvio', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDEnvio', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'esInicial',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'multiplicidadMinima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'multiplicidadMaxima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'detallesPaso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                     'numeroDePaso',
                'kind':                     'Data',
                'type':                     'Number',
                'optional':                 True,
                'exclude_from_values_form': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosEnviados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosDestinatarios',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnteriores',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_views':            [ 'Textual', ],
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientes',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientesEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'destinatarios',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'artefactosEnviados',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'recepciones',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'titulosDestinatarios', 'titulosArtefactosRecibidos', ],
                'related_types': [
                    {   'portal_types': [ 'BPDRecepcion', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosSiguientes',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosSiguientesEnCasoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name': 'ejecutores',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosAnteriores',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'requeridoEnPlazos',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDPlazo', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'aggregation_name':     'usosArtefactos',
                'columns':              [ 'title', 'description', 'esRequerido', 'modoDeUso', 'titulosArtefactosUsados', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosCaracteristicas',
                'columns':              [ 'title', 'description', 'esRequerida', 'modoDeUso', 'titulosCaracteristicasUsadas', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDExitoFinal', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDExitoFinal', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'detallesPaso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                     'numeroDePaso',
                'kind':                     'Data',
                'type':                     'Number',
                'optional':                 True,
                'exclude_from_values_form': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnteriores',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_views':            [ 'Textual', ],
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name':    'pasosAnteriores',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name': 'reglasDeNegocioAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosArtefactos',
                'columns':              [ 'title', 'description', 'esRequerido', 'modoDeUso', 'titulosArtefactosUsados', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosCaracteristicas',
                'columns':              [ 'title', 'description', 'esRequerida', 'modoDeUso', 'titulosCaracteristicasUsadas', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDExtensionProceso', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDExtensionProceso', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'condicion',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosProcesosExtendidos',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'puntosExtension',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', ],
                'related_types': [
                    {   'portal_types': [ 'BPDPuntoExtension', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDFracasoFinal', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDFracasoFinal', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'detallesPaso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                     'numeroDePaso',
                'kind':                     'Data',
                'type':                     'Number',
                'optional':                 True,
                'exclude_from_values_form': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnteriores',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_views':            [ 'Textual', ],
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name':    'pasosAnteriores',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name': 'reglasDeNegocioAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosArtefactos',
                'columns':              [ 'title', 'description', 'esRequerido', 'modoDeUso', 'titulosArtefactosUsados', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosCaracteristicas',
                'columns':              [ 'title', 'description', 'esRequerida', 'modoDeUso', 'titulosCaracteristicasUsadas', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDHerramienta', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDHerramienta', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                               'codigo',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'estado',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':               'fechaAdopcion',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':               'fechaObsolescencia',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':                               'nivelDeImposicion',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'version',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'coleccionesHerramientas',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionHerramientas', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'instrucciones',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', ],
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'responsablesDeHerramienta',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'artefactosDeHerramienta',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosAsistidos',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'pasosAsistidos',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'politicasDeNegocioGobernantes',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioDirigentes',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDOrganizacion', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDOrganizacion', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':      'permiteLeer',
                'kind':      'Data',
                'type':      'Boolean',
                'optional':  True,
                'read_only': True,
            },
            {   'name':      'permiteModificar',
                'kind':      'Data',
                'type':      'Boolean',
                'optional':  True,
                'read_only': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'abreviatura',
                'kind':     'Data',
                'type':     'string',
                'optional': True,
            },
            {   'name':     'responsabilidadesClave',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'coleccionesPoliticasDeNegocio',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionPoliticasDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesReglasDeNegocio',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionReglasDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesUnidadesOrganizacionales',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionUnidadesOrganizacionales', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesPerfiles',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionPerfiles', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesProcesosDeNegocio',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionProcesosDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesArtefactos',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionArtefactos', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesHerramientas',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionHerramientas', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosSupervisados',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosEjecutados',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'pasosEjecutados',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'detallesPaso', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDPasoSimple', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'remitenteDeRecepciones',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'tituloRemitente', 'titulosArtefactosRecibidos', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDRecepcion', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'destinatarioDeEnvios',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'titulosArtefactosEnviados', 'titulosDestinatarios', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDEnvio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'responsableDeArtefactos',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'responsableDeHerramientas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'politicasDeNegocioGobernantes',
                'columns':       [ 'title', 'description', 'codigo', 'estado', 'nivelDeImposicion', ],
                'related_types': [
                    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioDirigentes',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDPasoGestorExcepciones', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDPasoGestorExcepciones', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
        ],
    },
    {   'portal_types': [ 'BPDPasoSimple', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDPasoSimple', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'esInicial',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'detallesPaso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                     'numeroDePaso',
                'kind':                     'Data',
                'type':                     'Number',
                'optional':                 True,
                'exclude_from_values_form': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnteriores',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_views':            [ 'Textual', ],
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientes',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientesEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name':    'pasosSiguientes',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosSiguientesEnCasoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name': 'ejecutores',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosAnteriores',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'requeridoEnPlazos',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDPlazo', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'aggregation_name':     'usosArtefactos',
                'columns':              [ 'title', 'description', 'esRequerido', 'modoDeUso', 'titulosArtefactosUsados', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosCaracteristicas',
                'columns':              [ 'title', 'description', 'esRequerida', 'modoDeUso', 'titulosCaracteristicasUsadas', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDPerfil', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDPerfil', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'abreviatura',
                'kind':     'Data',
                'type':     'string',
                'optional': True,
            },
            {   'name':     'conocimientosPoseidos',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':     'habilidadesDemostradas',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':     'responsabilidadesClave',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':     'seleccionDePersonal',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'procesosSupervisados',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosEjecutados',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'pasosEjecutados',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'detallesPaso', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDPasoSimple', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'remitenteDeRecepciones',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'tituloRemitente', 'titulosArtefactosRecibidos', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDRecepcion', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'destinatarioDeEnvios',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'titulosArtefactosEnviados', 'titulosDestinatarios', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDEnvio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'responsableDeArtefactos',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'responsableDeHerramientas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'politicasDeNegocioGobernantes',
                'columns':       [ 'title', 'description', 'codigo', 'estado', 'nivelDeImposicion', ],
                'related_types': [
                    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioDirigentes',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDPlazo', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDPlazo', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'esInicial',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'agotarPlazo',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'tiempoDeEspera',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'unidadDeTiempo',
                'kind':     'Data',
                'type':     'selection',
                'optional': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'detallesPaso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                     'numeroDePaso',
                'kind':                     'Data',
                'type':                     'Number',
                'optional':                 True,
                'exclude_from_values_form': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnteriores',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_views':            [ 'Textual', ],
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientes',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientesEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name':    'pasosSiguientes',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosRequeridos',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDPasoSimple', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosSiguientesEnCasoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name': 'reglasDeNegocioAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosAnteriores',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'aggregation_name':     'usosArtefactos',
                'columns':              [ 'title', 'description', 'esRequerido', 'modoDeUso', 'titulosArtefactosUsados', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosCaracteristicas',
                'columns':              [ 'title', 'description', 'esRequerida', 'modoDeUso', 'titulosCaracteristicasUsadas', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                               'codigo',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'estado',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':               'fechaAdopcion',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':               'fechaObsolescencia',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':                               'nivelDeImposicion',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'version',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'coleccionesPoliticasDeNegocio',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionPoliticasDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioDerivadas',
                'columns':       [ 'title', 'description', 'codigo', 'estado', 'nivelDeImposicion', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosDeNegocioGobernados',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'participantesGobernados',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasGobernadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'artefactosGobernados',
                'columns':       [ 'title', 'description', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'proposito',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':     'preCondicion',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':     'postCondicion',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':     'responsableMantenimiento',
                'kind':     'Data',
                'type':     'string',
                'optional': True,
            },
            {   'name':     'seleccionDePersonal',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                               'esPlaneado',
                'kind':                               'Data',
                'type':                               'Boolean',
                'optional':                           True,
                'exclude_from_values_paragraph_when': False,
            },
            {   'name':                               'esContinuo',
                'kind':                               'Data',
                'type':                               'Boolean',
                'optional':                           True,
                'exclude_from_values_paragraph_when': False,
            },
            {   'name':                               'esRepetible',
                'kind':                               'Data',
                'type':                               'Boolean',
                'optional':                           True,
                'exclude_from_values_paragraph_when': False,
            },
            {   'name':                               'codigo',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'estado',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'nivelDeImposicion',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'version',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':               'fechaAdopcion',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':               'fechaObsolescencia',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':                          'detallesProceso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'coleccionesProcesosDeNegocio',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionProcesosDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'supervisor',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'ejecutores',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'extensiones',
                'columns':              [ 'title', 'description', 'condicion', 'titulosProcesosExtendidos', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDExtensionProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesEntradas',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionEntradas', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesSalidas',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionSalidas', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesPasos',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionPasos', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'politicasDeNegocioGobernantes',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioDirigentes',
                'columns':       [ 'title', 'description', 'codigo', 'estado', 'nivelDeImposicion', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasDocumentadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'artefactosGestionados',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'caracteristicasGestionadas',
                'columns':       [ 'title', 'description', 'claseCaracteristica', 'tituloArtefacto', ],
                'related_types': [
                    {   'portal_types': [ 'BPDCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'usadoComoSubProcesos',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDPuntoExtension', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDPuntoExtension', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'esInicial',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'detallesPaso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                     'numeroDePaso',
                'kind':                     'Data',
                'type':                     'Number',
                'optional':                 True,
                'exclude_from_values_form': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnteriores',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_views':            [ 'Textual', ],
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientes',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientesEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosProcesosExtensiones',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'extensionesProcesos',
                'columns':       [ 'title', 'description', 'condicion', 'tituloProcesoDeNegocio', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDExtensionProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosSiguientes',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosSiguientesEnCasoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name': 'reglasDeNegocioAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosAnteriores',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'aggregation_name':     'usosArtefactos',
                'columns':              [ 'title', 'description', 'esRequerido', 'modoDeUso', 'titulosArtefactosUsados', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosCaracteristicas',
                'columns':              [ 'title', 'description', 'esRequerida', 'modoDeUso', 'titulosCaracteristicasUsadas', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDRecepcion', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDRecepcion', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'esInicial',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'multiplicidadMinima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'multiplicidadMaxima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'condicion',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'detallesPaso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                     'numeroDePaso',
                'kind':                     'Data',
                'type':                     'Number',
                'optional':                 True,
                'exclude_from_values_form': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'tituloRemitente',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosRecibidos',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnteriores',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_views':            [ 'Textual', ],
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientes',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientesEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'remitente',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'artefactosRecibidos',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'envios',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'titulosDestinatarios', 'titulosArtefactosEnviados', ],
                'related_types': [
                    {   'portal_types': [ 'BPDEnvio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosSiguientes',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosSiguientesEnCasoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name': 'ejecutores',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosAnteriores',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'requeridoEnPlazos',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDPlazo', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'aggregation_name':     'usosArtefactos',
                'columns':              [ 'title', 'description', 'esRequerido', 'modoDeUso', 'titulosArtefactosUsados', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosCaracteristicas',
                'columns':              [ 'title', 'description', 'esRequerida', 'modoDeUso', 'titulosCaracteristicasUsadas', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDReferenciaCualificada', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDReferenciaCualificada', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDReglaDeNegocio', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDReglaDeNegocio', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                               'codigo',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'estado',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':               'fechaAdopcion',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':               'fechaObsolescencia',
                'kind':               'Data',
                'type':               'Date',
                'optional':           True,
                'exclude_from_views': [ 'Textual', ],
            },
            {   'name':                               'nivelDeImposicion',
                'kind':                               'Data',
                'type':                               'selection',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
            {   'name':                               'version',
                'kind':                               'Data',
                'type':                               'string',
                'optional':                           True,
                'exclude_from_views':                 [ 'Textual', ],
                'exclude_from_values_paragraph_when': '',
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'coleccionesReglasDeNegocio',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionReglasDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'politicasDeNegocioBase',
                'columns':       [ 'title', 'description', 'codigo', 'estado', 'nivelDeImposicion', ],
                'related_types': [
                    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosDeNegocioDirigidos',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'pasosAplicandoLaRegla',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'detallesPaso', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'participantesDirigidos',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'artefactosDirigidos',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasDirigidas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDSalida', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDSalida', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'multiplicidadMinima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'multiplicidadMaxima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'esRequerida',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosDeSalida',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'artefactosDeSalida',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDSubProceso', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDSubProceso', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'esInicial',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'detallesPaso',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                     'numeroDePaso',
                'kind':                     'Data',
                'type':                     'Number',
                'optional':                 True,
                'exclude_from_values_form': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'tituloProcesoUsado',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnteriores',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_views':            [ 'Textual', ],
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosAnterioresEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientes',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosPasosSiguientesEnCasoExcepcion',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'procesoUsado',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', ],
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosSiguientes',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosSiguientesEnCasoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name': 'ejecutores',
                'columns':       [ 'title', 'description', 'abreviatura', ],
                'related_types': [
                    {   'portal_types': [ 'BPDOrganizacion', 'BPDPerfil', 'BPDUnidadOrganizacional', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'herramientasAplicadas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name':    'pasosAnteriores',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'pasosOriginandoExcepcion',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDExitoFinal', 'BPDFracasoFinal', 'BPDPasoGestorExcepciones', 'BPDPasoSimple', 'BPDPlazo', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'relation_name':    'requeridoEnPlazos',
                'columns':          [ 'title', 'description', 'detallesPaso', ],
                'related_types':    [
                    {   'portal_types': [ 'BPDPlazo', ],
                        'reuse_config': 'Default',
                    },
                ],
                'candidates_scope': 'Owner',
            },
            {   'aggregation_name':     'usosArtefactos',
                'columns':              [ 'title', 'description', 'esRequerido', 'modoDeUso', 'titulosArtefactosUsados', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'usosCaracteristicas',
                'columns':              [ 'title', 'description', 'esRequerida', 'modoDeUso', 'titulosCaracteristicasUsadas', ],
                'contains_collections': False,
                'tabular_tree':         False,
                'subitems':             [
                    {   'portal_types': [ 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDUnidadOrganizacional', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDUnidadOrganizacional', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'abreviatura',
                'kind':     'Data',
                'type':     'string',
                'optional': True,
            },
            {   'name':     'responsabilidadesClave',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
        'traversals':   [
            {   'aggregation_name':     'coleccionesUnidadesOrganizacionales',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionUnidadesOrganizacionales', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'aggregation_name':     'coleccionesPerfiles',
                'columns':              [ 'title', 'description', ],
                'contains_collections': True,
                'tabular_tree':         True,
                'subitems':             [
                    {   'portal_types': [ 'BPDColeccionPerfiles', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosSupervisados',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'procesosEjecutados',
                'columns':       [ 'title', 'description', 'proposito', 'detallesProceso', 'codigo', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDProcesoDeNegocioSimple', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'pasosEjecutados',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'detallesPaso', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDDecision', 'BPDEnvio', 'BPDPasoSimple', 'BPDRecepcion', 'BPDSubProceso', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'remitenteDeRecepciones',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'tituloRemitente', 'titulosArtefactosRecibidos', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDRecepcion', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'destinatarioDeEnvios',
                'columns':       [ 'title', 'description', 'tituloProcesoDeNegocio', 'titulosArtefactosEnviados', 'titulosDestinatarios', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDEnvio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'responsableDeArtefactos',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'responsableDeHerramientas',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDHerramienta', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'politicasDeNegocioGobernantes',
                'columns':       [ 'title', 'description', 'codigo', 'estado', 'nivelDeImposicion', ],
                'related_types': [
                    {   'portal_types': [ 'BPDPoliticaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'reglasDeNegocioDirigentes',
                'columns':       [ 'title', 'description', 'codigo', ],
                'related_types': [
                    {   'portal_types': [ 'BPDReglaDeNegocio', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDUsoArtefacto', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDUsoArtefacto', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'esRequerido',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'modoDeUso',
                'kind':     'Data',
                'type':     'selection',
                'optional': True,
            },
            {   'name':     'multiplicidadMinima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'multiplicidadMaxima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':                          'tituloPasoDeProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosArtefactosUsados',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'artefactosUsados',
                'columns':       [ 'title', 'description', 'codigo', 'estado', 'nivelDeImposicion', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
    {   'portal_types': [ 'BPDUsoCaracteristica', ],
        'config_name':  'Mention',
        'mode':         'Reference',
        'extension':    'OneParagraph',
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
        ],
    },
    {   'portal_types': [ 'BPDUsoCaracteristica', ],
        'extension':    'NonTextAttrsOneParagraph',
        'divider':      [ 'Before', ],
        'attrs':        [
            {   'name': 'title',
                'kind': 'Data',
                'type': 'String',
            },
            {   'name': 'archetype_name',
                'kind': 'Meta',
                'type': 'String',
            },
            {   'name':     'description',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':       'text',
                'kind':       'Data',
                'type':       'Text',
                'optional':   True,
                'hide_label': True,
            },
            {   'name':     'ownerName',
                'kind':     'Data',
                'type':     'Text',
                'optional': True,
            },
            {   'name':                          'qualifiedName',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':     'esRequerida',
                'kind':     'Data',
                'type':     'Boolean',
                'optional': True,
            },
            {   'name':     'modoDeUso',
                'kind':     'Data',
                'type':     'selection',
                'optional': True,
            },
            {   'name':     'multiplicidadMinima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':     'multiplicidadMaxima',
                'kind':     'Data',
                'type':     'Number',
                'optional': True,
            },
            {   'name':                          'tituloPasoDeProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'tituloProcesoDeNegocio',
                'kind':                          'Data',
                'type':                          'string',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
            {   'name':                          'titulosCaracteristicasUsadas',
                'kind':                          'Data',
                'type':                          '',
                'optional':                      True,
                'exclude_from_values_form':      True,
                'exclude_from_values_paragraph': True,
            },
        ],
        'traversals':   [
            {   'relation_name': 'caracteristicasUsadas',
                'columns':       [ 'title', 'description', 'claseCaracteristica', 'tituloArtefacto', ],
                'related_types': [
                    {   'portal_types': [ 'BPDCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referidos',
                'columns':       [ 'title', 'description', ],
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
            {   'relation_name': 'referentes',
                'columns':       [ 'title', 'description', ],
                'dependency':    'Supplier',
                'related_types': [
                    {   'portal_types': [ 'BPDArtefacto', 'BPDCaracteristica', 'BPDDecision', 'BPDEntrada', 'BPDEnvio', 'BPDExitoFinal', 'BPDExtensionProceso', 'BPDFracasoFinal', 'BPDHerramienta', 'BPDOrganizacion', 'BPDPasoSimple', 'BPDPerfil', 'BPDPlazo', 'BPDPoliticaDeNegocio', 'BPDProcesoDeNegocioSimple', 'BPDPuntoExtension', 'BPDRecepcion', 'BPDReglaDeNegocio', 'BPDSalida', 'BPDSubProceso', 'BPDUnidadOrganizacional', 'BPDUsoArtefacto', 'BPDUsoCaracteristica', ],
                        'reuse_config': 'Default',
                    },
                ],
            },
        ],
    },
]
